@@ -42,7 +42,7 @@ static krb5_error_code ipadb_get_delegation_acl(krb5_context kcontext,
 {
     struct ipadb_context *ipactx;
     krb5_error_code kerr;
-    char *filter = NULL;
+    char *filter = NULL, *basedn = NULL;
     int ret;
 
     ipactx = ipadb_get_context(kcontext);
@@ -58,12 +58,20 @@ static krb5_error_code ipadb_get_delegation_acl(krb5_context kcontext,
         goto done;
     }
 
+    ret = asprintf(&basedn,
+                   "cn=s4u2proxy,cn=etc,%s", ipactx->base);
+    if (ret == -1) {
+        kerr = ENOMEM;
+        goto done;
+    }
+
     /* == Search ACL info == */
-    kerr = ipadb_deref_search(ipactx, ipactx->base,
+    kerr = ipadb_deref_search(ipactx, basedn,
                               LDAP_SCOPE_SUBTREE, filter, acl_attrs,
                               search_attrs, acl_attrs, results);
 
 done:
+    free(basedn);
     free(filter);
     return kerr;
 }
@@ -284,3 +292,44 @@ done:
     ldap_msgfree(res);
     return kerr;
 }
+
+
+krb5_error_code ipadb_allowed_to_delegate_from(krb5_context context,
+                                               krb5_const_principal client,
+                                               krb5_const_principal server,
+                                               krb5_pac server_pac,
+                                               const krb5_db_entry *proxy)
+{
+    char **acl_list = NULL;
+    krb5_error_code kerr;
+    size_t i;
+
+    kerr = ipadb_get_tl_data((krb5_db_entry *) proxy, KRB5_TL_CONSTRAINED_DELEGATION_ACL,
+                             sizeof(acl_list), (krb5_octet *)&acl_list);
+
+    if (kerr != 0 && kerr != ENOENT) {
+        return KRB5KDC_ERR_BADOPTION;
+    }
+
+    if (kerr == ENOENT) {
+        return KRB5_PLUGIN_OP_NOTSUPP;
+    }
+
+    kerr = KRB5KDC_ERR_BADOPTION;
+    if (acl_list != NULL) {
+        krb5_principal acl;
+        for (i = 0; acl_list[i] != NULL; i++) {
+            if (krb5_parse_name(context, acl_list[i], &acl) != 0)
+                continue;
+            if ((server == NULL) ||
+                (krb5_principal_compare(context, server, acl) == TRUE)) {
+                kerr = 0;
+                krb5_free_principal(context, acl);
+                break;
+            }
+            krb5_free_principal(context, acl);
+        }
+    }
+
+    return kerr;
+}
