@@ -2,6 +2,7 @@
 # Copyright (C) 2016  FreeIPA Contributors see COPYING for license
 #
 
+from collections.abc import MutableMapping
 import errno
 import json
 import locale
@@ -9,21 +10,12 @@ import logging
 import os
 import time
 
-import six
-
 from . import compat
 from . import schema
 from ipaclient.plugins.rpcclient import rpcclient
 from ipalib.constants import USER_CACHE_PATH
 from ipapython.dnsutil import DNSName
 
-# pylint: disable=no-name-in-module, import-error
-if six.PY3:
-    from collections.abc import MutableMapping
-else:
-    from collections import MutableMapping
-# pylint: enable=no-name-in-module, import-error
-
 logger = logging.getLogger(__name__)
 
 
@@ -41,7 +33,7 @@ class ServerInfo(MutableMapping):
         try:
             self._language = locale.setlocale(
                 locale.LC_MESSAGES, ''
-            ).split('.')[0].lower()
+            ).split('.', maxsplit=1)[0].lower()
         except locale.Error:
             self._language = 'en_us'
 
@@ -117,9 +109,12 @@ class ServerInfo(MutableMapping):
 
 def get_package(api):
     if api.env.in_tree:
+        # server packages are not published on pypi.org
+        # pylint: disable=useless-suppression
         # pylint: disable=import-error,ipa-forbidden-import
         from ipaserver import plugins
         # pylint: enable=import-error,ipa-forbidden-import
+        # pylint: enable=useless-suppression
     else:
         try:
             plugins = api._remote_plugins
