@@ -921,7 +921,10 @@ from ipalib.backend import Backend
 from ipalib.frontend import Command, LocalOrRemote, Updater
 from ipalib.frontend import Object, Method
 from ipalib.crud import Create, Retrieve, Update, Delete, Search
-from ipalib.parameters import DefaultFrom, Bool, Flag, Int, Decimal, Bytes, Str, IA5Str, Password, DNParam
+from ipalib.parameters import (
+    DefaultFrom, Bool, Flag, Int, Decimal, Bytes, Str, IA5Str,
+    Password, DNParam, SerialNumber
+)
 from ipalib.parameters import (BytesEnum, StrEnum, IntEnum, AccessTime, File,
                         DateTime, DNSNameParam)
 from ipalib.errors import SkipPluginModule
@@ -946,7 +949,7 @@ class API(plugable.API):
                 ipaserver.plugins,
             )
         else:
-            import ipaclient.remote_plugins
+            import ipaclient.remote_plugins  # pylint: disable=cyclic-import
             import ipaclient.plugins
             result = (
                 ipaclient.remote_plugins.get_package(self),
