@@ -52,7 +52,7 @@ import six
 
 from ipalib.backend import Connectible
 from ipalib.constants import LDAP_GENERALIZED_TIME_FORMAT
-from ipalib.errors import (public_errors, UnknownError, NetworkError,
+from ipalib.errors import (errors_by_code, UnknownError, NetworkError,
                            XMLRPCMarshallError, JSONError)
 from ipalib import errors, capabilities
 from ipalib.request import context, Connection
@@ -77,7 +77,6 @@ try:
     from xmlrpclib import (Binary, Fault, DateTime, dumps, loads, ServerProxy,
             Transport, ProtocolError, MININT, MAXINT)
 except ImportError:
-    # pylint: disable=import-error
     from xmlrpc.client import (Binary, Fault, DateTime, dumps, loads, ServerProxy,
             Transport, ProtocolError, MININT, MAXINT)
 
@@ -97,8 +96,6 @@ logger = logging.getLogger(__name__)
 COOKIE_NAME = 'ipa_session'
 CCACHE_COOKIE_KEY = 'X-IPA-Session-Cookie'
 
-errors_by_code = dict((e.errno, e) for e in public_errors)
-
 
 def update_persistent_client_session_data(principal, data):
     '''
@@ -284,7 +281,7 @@ class _JSONPrimer(dict):
 
     Prepare a data structure for JSON serialization. In an ideal world, priming
     could be handled by the default hook of json.dumps(). Unfortunately the
-    hook treats Python 2 str as text while FreeIPA considers str as bytes.
+    hook treats Python 2 str as text while IPA considers str as bytes.
 
     The primer uses a couple of tricks to archive maximum performance:
 
@@ -535,7 +532,7 @@ class LanguageAwareTransport(MultiProtocolTransport):
         try:
             lang = locale.setlocale(
                 locale.LC_MESSAGES, ''
-            ).split('.')[0].lower()
+            ).split('.', maxsplit=1)[0].lower()
         except locale.Error:
             # fallback to default locale
             lang = 'en_us'
@@ -686,6 +683,9 @@ class KerbTransport(SSLTransport):
             return False
         return True
 
+    # pylint: disable=inconsistent-return-statements
+    # pylint does not properly manage the _handle_exception call
+    # that is always raising an exception
     def single_request(self, host, handler, request_body, verbose=0):
         # Based on Python 2.7's xmllib.Transport.single_request
         try:
@@ -741,12 +741,13 @@ class KerbTransport(SSLTransport):
             self.close()
             logger.debug("HTTP server has closed connection (%s)", host)
             raise
-        except BaseException as e:
+        except BaseException:
             # Unexpected exception may leave connections in a bad state.
             self.close()
             logger.debug("HTTP connection destroyed (%s)",
                          host, exc_info=True)
             raise
+    # pylint: enable=inconsistent-return-statements
 
     if six.PY3:
         def __send_request(self, connection, host, handler, request_body, debug):
@@ -761,7 +762,7 @@ class KerbTransport(SSLTransport):
             else:
                 connection.putrequest("POST", handler)
             headers.append(("User-Agent", self.user_agent))
-            self.send_headers(connection, headers)  # pylint: disable=E1101
+            self.send_headers(connection, headers)
             self.send_content(connection, request_body)
             return connection
 
@@ -835,7 +836,7 @@ class KerbTransport(SSLTransport):
                      cookie_string, principal)
         try:
             update_persistent_client_session_data(principal, cookie_string)
-        except Exception as e:
+        except Exception:
             # Not fatal, we just can't use the session cookie we were sent.
             pass
 
@@ -975,7 +976,7 @@ class RPCClient(Connectible):
                          principal, e)
             try:
                 delete_persistent_client_session_data(principal)
-            except Exception as e:
+            except Exception:
                 pass
             return original_url
         except Cookie.URLMismatch as e:
@@ -994,8 +995,6 @@ class RPCClient(Connectible):
         # Form the session URL by substituting the session path into the original URL
         scheme, netloc, path, params, query, fragment = urllib.parse.urlparse(original_url)
         path = self.session_path
-        # urlencode *can* take one argument
-        # pylint: disable=too-many-function-args
         session_url = urllib.parse.urlunparse((scheme, netloc, path, params, query, fragment))
 
         return session_url
@@ -1076,11 +1075,9 @@ class RPCClient(Connectible):
                             )
                     # We don't care about the response, just that we got one
                     return serverproxy
-                # pylint: disable=try-except-raise
                 except errors.KerberosError:
                     # kerberos error on one server is likely on all
                     raise
-                # pylint: enable=try-except-raise
                 except ProtocolError as e:
                     if hasattr(context, 'session_cookie') and e.errcode == 401:
                         # Unauthorized. Remove the session and try again.
@@ -1170,11 +1167,11 @@ class RPCClient(Connectible):
                     try:
                         principal = getattr(context, 'principal', None)
                         delete_persistent_client_session_data(principal)
-                    except Exception as e:
+                    except Exception as e2:
                         # This shouldn't happen if we have a session
                         # but it isn't fatal.
                         logger.debug("Error trying to remove persisent "
-                                     "session data: %s", e)
+                                     "session data: %s", e2)
 
                     # Create a new serverproxy with the non-session URI
                     serverproxy = self.create_connection(
