@@ -26,10 +26,11 @@ import io
 import pwd
 import grp
 import re
+import shutil
 import stat
 import tempfile
+from ctypes.util import find_library
 from tempfile import NamedTemporaryFile
-import shutil
 
 import cryptography.x509
 
@@ -73,6 +74,10 @@ TRUSTED_PEER_TRUST_FLAGS = TrustFlags(
 )
 
 
+def nss_supports_dbm():
+    return bool(find_library("nssdbm3"))
+
+
 def get_ca_nickname(realm, format=CA_NICKNAME_FMT):
     return format % realm
 
@@ -251,19 +256,30 @@ class NSSDatabase:
     # BaseCertDB is a class that knows nothing about IPA.
     # Generic NSS DB code should be moved here.
 
-    def __init__(self, nssdir=None, dbtype='auto'):
+    def __init__(self, nssdir=None, dbtype='auto', token=None, pwd_file=None):
+        if nssdir is not None:
+            self.secdir = nssdir
+            self._is_temporary = False
+            if dbtype == "auto":
+                dbtype = self._detect_dbtype()
+
+        if dbtype == "dbm" and not nss_supports_dbm():
+            raise ValueError(
+                f"NSS is built without support of the legacy database(DBM) "
+                f"directory '{nssdir}'",
+            )
+
         if nssdir is None:
             self.secdir = tempfile.mkdtemp()
             self._is_temporary = True
-        else:
-            self.secdir = nssdir
-            self._is_temporary = False
-            if dbtype == 'auto':
-                dbtype = self._detect_dbtype()
 
-        self.pwd_file = os.path.join(self.secdir, 'pwdfile.txt')
+        if pwd_file is None:
+            self.pwd_file = os.path.join(self.secdir, 'pwdfile.txt')
+        else:
+            self.pwd_file = pwd_file
         self.dbtype = None
         self.certdb = self.keydb = self.secmod = None
+        self.token = token
         # files in actual db
         self.filenames = ()
         # all files that are handled by create_db(backup=True)
@@ -328,6 +344,8 @@ class NSSDatabase:
             "-d", '{}:{}'.format(self.dbtype, self.secdir)
         ]
         new_args.extend(args)
+        if self.token:
+            new_args.extend(["-h", self.token])
         new_args.extend(['-f', self.pwd_file])
         # When certutil makes a request it creates a file in the cwd, make
         # sure we are in a unique place when this happens.
@@ -599,7 +617,7 @@ class NSSDatabase:
         try:
             self.run_pk12util(args)
         except ipautil.CalledProcessError as e:
-            if e.returncode == 17 or e.returncode == 18:
+            if e.returncode in (17, 18):
                 raise Pkcs12ImportIncorrectPasswordError(
                     "incorrect password for pkcs#12 file %s" % pkcs12_filename)
             elif e.returncode == 10:
@@ -664,6 +682,9 @@ class NSSDatabase:
                                     "%s",
                                     filename, line, e)
                                 continue
+                            logger.error('Failed to load certificate in %s '
+                                         'at line %s: %s',
+                                         filename, line, e)
                         else:
                             extracted_certs.append(cert)
                             loaded = True
