@@ -53,12 +53,7 @@ from ipapython.dn import DN, RDN
 from ipapython.dnsutil import DNSName
 from ipapython.kerberos import Principal
 
-# pylint: disable=no-name-in-module, import-error
-if six.PY3:
-    from collections.abc import MutableMapping
-else:
-    from collections import MutableMapping
-# pylint: enable=no-name-in-module, import-error
+from collections.abc import MutableMapping
 
 if six.PY3:
     unicode = str
@@ -89,7 +84,7 @@ if six.PY2 and hasattr(ldap, 'LDAPBytesWarning'):
     # XXX silence python-ldap's BytesWarnings
     warnings.filterwarnings(
         action="ignore",
-        category=ldap.LDAPBytesWarning,  # pylint: disable=no-member
+        category=ldap.LDAPBytesWarning,
     )
 
 
@@ -260,7 +255,7 @@ class LDAPEntry(MutableMapping):
 
         Keyword arguments can be used to override values of specific attributes.
         """
-        super(LDAPEntry, self).__init__()  # pylint: disable=no-member
+        super(LDAPEntry, self).__init__()
 
         if isinstance(_conn, LDAPEntry):
             assert _dn is None
@@ -289,7 +284,6 @@ class LDAPEntry(MutableMapping):
         self._single_value_view = None
 
         if isinstance(_obj, LDAPEntry):
-            #pylint: disable=E1103
             self._not_list = set(_obj._not_list)
             self._orig_raw = dict(_obj._orig_raw)
             if _obj.conn is _conn:
@@ -689,6 +683,7 @@ class LDAPClient:
         '1.3.6.1.4.1.1466.115.121.1.1'   : bytes, # ACI item
         '1.3.6.1.4.1.1466.115.121.1.4'   : bytes, # Audio
         '1.3.6.1.4.1.1466.115.121.1.5'   : bytes, # Binary
+        '1.3.6.1.4.1.1466.115.121.1.7'   : bool,  # Boolean
         '1.3.6.1.4.1.1466.115.121.1.8'   : bytes, # Certificate
         '1.3.6.1.4.1.1466.115.121.1.9'   : bytes, # Certificate List
         '1.3.6.1.4.1.1466.115.121.1.10'  : bytes, # Certificate Pair
@@ -763,6 +758,8 @@ class LDAPClient:
         'nsslapd-enable-upgrade-hash': True,
         'nsslapd-db-locks': True,
         'nsslapd-logging-hr-timestamps-enabled': True,
+        'nsslapd-ldapientrysearchbase': True,
+        'nsslapd-ldapidnmappingbase': True,
         'nsslapd-sizelimit': True,
     })
 
@@ -1013,6 +1010,8 @@ class LDAPClient:
                     return val
                 elif target_type is unicode:
                     return val.decode('utf-8')
+                elif target_type is bool:
+                    return val.decode('utf-8') == 'TRUE'
                 elif target_type is datetime.datetime:
                     return datetime.datetime.strptime(
                         val.decode('utf-8'), LDAP_GENERALIZED_TIME_FORMAT)
@@ -1083,6 +1082,7 @@ class LDAPClient:
     def error_handler(self, arg_desc=None):
         """Context manager that handles LDAPErrors
         """
+        desc = None
         try:
             try:
                 yield
@@ -1769,15 +1769,18 @@ class LDAPCache(LDAPClient):
 
     def __init__(self, ldap_uri, start_tls=False, force_schema_updates=False,
                  no_schema=False, decode_attrs=True, cacert=None,
-                 sasl_nocanon=True, enable_cache=True, cache_size=100):
+                 sasl_nocanon=True, enable_cache=True, cache_size=100,
+                 debug_cache=False):
 
         self.cache = OrderedDict()
         self._enable_cache = True  # initialize to zero to satisfy pylint
+        self._debug_cache = False  # initialize to zero to satisfy pylint
 
         object.__setattr__(self, '_cache_misses', 0)
         object.__setattr__(self, '_cache_hits', 0)
         object.__setattr__(self, '_enable_cache',
                            enable_cache and cache_size > 0)
+        object.__setattr__(self, '_debug_cache', debug_cache)
         object.__setattr__(self, '_cache_size', cache_size)
 
         super(LDAPCache, self).__init__(
@@ -1798,7 +1801,7 @@ class LDAPCache(LDAPClient):
         return self._cache_size  # pylint: disable=no-member
 
     def emit(self, msg, *args, **kwargs):
-        if self._enable_cache:
+        if self._enable_cache and self._debug_cache:
             logger.debug(msg, *args, **kwargs)
 
     def copy_entry(self, dn, entry, attrs=[]):
@@ -1936,7 +1939,6 @@ class LDAPCache(LDAPClient):
 
     def get_entry(self, dn, attrs_list=None, time_limit=None,
                   size_limit=None, get_effective_rights=False):
-        # pylint: disable=no-member
         if not self._enable_cache:
             return super(LDAPCache, self).get_entry(
                 dn, attrs_list, time_limit, size_limit, get_effective_rights
@@ -1974,7 +1976,6 @@ class LDAPCache(LDAPClient):
             get_all = True
 
         if entry and entry.all and get_all:
-            # self.hit  # pylint: disable=pointless-statement
             hits = self._cache_hits + 1  # pylint: disable=no-member
             object.__setattr__(self, '_cache_hits', hits)
             self.cache_status('HIT')
