@@ -18,6 +18,7 @@ import sys
 import tempfile
 from contextlib import contextmanager
 from augeas import Augeas
+from pkg_resources import parse_version
 
 from ipalib import api, x509
 from ipalib.constants import RENEWAL_CA_NAME, RA_AGENT_PROFILE, IPA_CA_RECORD
@@ -36,6 +37,7 @@ from ipapython import ipautil, version
 from ipapython import ipaldap
 from ipapython import directivesetter
 from ipapython.dn import DN
+from ipapython.version import KRB5_BUILD_VERSION
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 from ipaserver import servroles
@@ -1523,6 +1525,9 @@ def upgrade_bind(fstore):
     else:
         bind_started = False
 
+    # create or update autobind entry
+    bind.setup_autobind()
+
     try:
         changed = bind.setup_named_conf(backup=True)
         if changed:
@@ -1958,6 +1963,9 @@ def upgrade_configuration():
     enable_server_snippet()
     setup_kpasswd_server(krb)
 
+    if KRB5_BUILD_VERSION >= parse_version('1.20'):
+        krb.pac_tkt_sign_support_enable()
+
     # Must be executed after certificate_renewal_update
     # (see function docstring for details)
     http_certificate_ensure_ipa_ca_dnsname(http)
