@@ -30,8 +30,9 @@ import cryptography.x509
 from cryptography.hazmat.primitives import hashes, serialization
 from dns import resolver, reversename
 import six
+import sys
 
-from ipalib import Command, Str, Int, Flag
+from ipalib import Command, Str, Int, Flag, StrEnum, SerialNumber
 from ipalib import api
 from ipalib import errors, messages
 from ipalib import x509
@@ -52,6 +53,7 @@ from ipalib.request import context
 from ipalib import output
 from ipapython import dnsutil, kerberos
 from ipapython.dn import DN
+from ipapython.ipautil import datetime_from_utctimestamp
 from ipaserver.plugins.service import normalize_principal, validate_realm
 from ipaserver.masters import (
     ENABLED_SERVICE, CONFIGURED_SERVICE, is_service_enabled
@@ -254,8 +256,9 @@ def normalize_pkidate(value):
 
 
 def convert_pkidatetime(value):
-    value = datetime.datetime.fromtimestamp(int(value) // 1000)
-    return x509.format_datetime(value)
+    if isinstance(value, str):
+        value = int(value)
+    return x509.format_datetime(datetime_from_utctimestamp(value, units=1000))
 
 
 def normalize_serial_number(num):
@@ -444,7 +447,7 @@ class BaseCertObject(Object):
             label=_('Fingerprint (SHA256)'),
             flags={'no_create', 'no_update', 'no_search'},
         ),
-        Int(
+        SerialNumber(
             'serial_number',
             label=_('Serial number'),
             doc=_('Serial number in decimal or if prefixed with 0x in hexadecimal'),
@@ -481,7 +484,8 @@ class BaseCertObject(Object):
                 base64.b64decode(obj['certificate']))
             obj['subject'] = DN(cert.subject)
             obj['issuer'] = DN(cert.issuer)
-            obj['serial_number'] = cert.serial_number
+            obj['serial_number'] = str(cert.serial_number)
+            obj['serial_number_hex'] = '0x%X' % cert.serial_number
             obj['valid_not_before'] = x509.format_datetime(
                     cert.not_valid_before)
             obj['valid_not_after'] = x509.format_datetime(
@@ -504,10 +508,6 @@ class BaseCertObject(Object):
                     logger.warning(
                         "Encountered bad GeneralName; skipping", exc_info=True)
 
-        serial_number = obj.get('serial_number')
-        if serial_number is not None:
-            obj['serial_number_hex'] = u'0x%X' % serial_number
-
     def _add_san_attribute(self, obj, full, gn):
         name_type_map = {
             cryptography.x509.RFC822Name:
@@ -569,20 +569,20 @@ class certreq(BaseCertObject):
             'request_type',
             default=u'pkcs10',
             autofill=True,
-            flags={'no_option', 'no_update', 'no_update', 'no_search'},
+            flags={'no_option', 'no_update', 'no_search'},
         ),
         Str(
             'profile_id?', validate_profile_id,
             label=_("Profile ID"),
             doc=_("Certificate Profile to use"),
-            flags={'no_update', 'no_update', 'no_search'},
+            flags={'no_update', 'no_search'},
         ),
         Str(
             'cert_request_status',
             label=_('Request status'),
             flags={'no_create', 'no_update', 'no_search'},
         ),
-        Int(
+        Str(
             'request_id',
             label=_('Request id'),
             primary_key=True,
@@ -873,7 +873,7 @@ class cert_request(Create, BaseCertMethod, VirtualCommand):
                             "with subject alt name '%s'.") % name)
                 if not bypass_caacl:
                     if principal_type == KRBTGT:
-                        ca_kdc_check(ldap, alt_principal.hostname)
+                        ca_kdc_check(self.api, alt_principal.hostname)
                     else:
                         caacl_check(alt_principal, ca, profile_id)
 
@@ -952,13 +952,13 @@ class cert_request(Create, BaseCertMethod, VirtualCommand):
                         reason=e,
                     )
                 )
-            result['request_id'] = int(result['request_id'])
+            result['request_id'] = result['request_id']
             result['cacn'] = ca_obj['cn'][0]
 
         # Success? Then add it to the principal's entry
         # (unless the profile tells us not to)
         profile = api.Command['certprofile_show'](profile_id)
-        store = profile['result']['ipacertprofilestoreissued'][0] == 'TRUE'
+        store = profile['result']['ipacertprofilestoreissued'][0]
         if store and 'certificate' in result:
             cert = result.get('certificate')
             kwargs = dict(addattr=u'usercertificate={}'.format(cert))
@@ -983,7 +983,7 @@ class cert_request(Create, BaseCertMethod, VirtualCommand):
 
         return dict(
             result=result,
-            value=pkey_to_value(int(result['request_id']), kw),
+            value=pkey_to_value(result['request_id'], kw),
         )
 
     def lookup_principal(self, principal):
@@ -1368,7 +1368,7 @@ class cert_show(Retrieve, CertMethod, VirtualCommand):
         # Dogtag lightweight CAs have shared serial number domain, so
         # we don't tell Dogtag the issuer (but we check the cert after).
         #
-        result = self.Backend.ra.get_certificate(str(serial_number))
+        result = self.Backend.ra.get_certificate(serial_number)
         cert = x509.load_der_x509_certificate(
                     base64.b64decode(result['certificate']))
 
@@ -1378,7 +1378,7 @@ class cert_show(Retrieve, CertMethod, VirtualCommand):
             logger.debug("Not granted by ACI to retrieve certificate, "
                          "looking at principal")
             if not bind_principal_can_manage_cert(cert):
-                raise acierr  # pylint: disable=E0702
+                raise acierr
 
         ca_obj = api.Command.ca_show(
             options['cacn'],
@@ -1441,7 +1441,7 @@ class cert_revoke(PKQuery, CertMethod, VirtualCommand):
 
         # Make sure that the cert specified by issuer+serial exists.
         # Will raise NotFound if it does not.
-        resp = api.Command.cert_show(unicode(serial_number), cacn=kw['cacn'])
+        resp = api.Command.cert_show(serial_number, cacn=kw['cacn'])
 
         try:
             self.check_access()
@@ -1463,7 +1463,8 @@ class cert_revoke(PKQuery, CertMethod, VirtualCommand):
             # we don't tell Dogtag the issuer (but we already checked that
             # the given serial was issued by the named ca).
             result=self.Backend.ra.revoke_certificate(
-                str(serial_number), revocation_reason=revocation_reason)
+                serial_number,
+                revocation_reason=revocation_reason)
         )
 
 
@@ -1487,7 +1488,8 @@ class cert_remove_hold(PKQuery, CertMethod, VirtualCommand):
             # we don't tell Dogtag the issuer (but we already checked that
             # the given serial was issued by the named ca).
             result=self.Backend.ra.take_certificate_off_hold(
-                str(serial_number))
+                serial_number
+            )
         )
 
 
@@ -1501,17 +1503,13 @@ class cert_find(Search, CertMethod):
             doc=_('Match cn attribute in subject'),
             autofill=False,
         ),
-        Int('min_serial_number?',
+        SerialNumber('min_serial_number?',
             doc=_("minimum serial number"),
             autofill=False,
-            minvalue=0,
-            maxvalue=2147483647,
         ),
-        Int('max_serial_number?',
+        SerialNumber('max_serial_number?',
             doc=_("maximum serial number"),
             autofill=False,
-            minvalue=0,
-            maxvalue=2147483647,
         ),
         Flag('exactly?',
             doc=_('match the common name exactly'),
@@ -1557,6 +1555,12 @@ class cert_find(Search, CertMethod):
             normalizer=normalize_pkidate,
             autofill=False,
         ),
+        StrEnum(
+            'status?',
+            doc=_("Status of the certificate"),
+            values=(u'VALID', u'INVALID', u'REVOKED', u'EXPIRED',
+                    u'REVOKED_EXPIRED'),
+        ),
         Flag('pkey_only?',
             label=_("Primary key only"),
             doc=_("Results should contain primary key attribute only "
@@ -1614,7 +1618,19 @@ class cert_find(Search, CertMethod):
             )
 
     def _get_cert_key(self, cert):
-        return (DN(cert.issuer), cert.serial_number)
+        # for cert-find with a certificate value
+        if isinstance(cert, x509.IPACertificate):
+            return (DN(cert.issuer), cert.serial_number)
+
+        issuer = []
+        for oid, value in cert.get_issuer().get_components():
+            issuer.append(
+                '{}={}'.format(oid.decode('utf-8'), value.decode('utf-8'))
+            )
+        issuer = ','.join(issuer)
+        # Use this to flip from OpenSSL reverse to X500 ordering
+        issuer = DN(issuer).x500_text()
+        return (DN(issuer), cert.get_serial_number())
 
     def _cert_search(self, pkey_only, **options):
         result = collections.OrderedDict()
@@ -1642,7 +1658,8 @@ class cert_find(Search, CertMethod):
                      'validnotafter_from', 'validnotafter_to',
                      'validnotbefore_from', 'validnotbefore_to',
                      'issuedon_from', 'issuedon_to',
-                     'revokedon_from', 'revokedon_to'):
+                     'revokedon_from', 'revokedon_to',
+                     'status'):
             try:
                 value = options[name]
             except KeyError:
@@ -1654,6 +1671,13 @@ class cert_find(Search, CertMethod):
             ra_options[name] = value
         if exactly:
             ra_options['exactly'] = True
+        if 'sizelimit' in options:
+            # sizelimit = 0 means return everything, drop it and let
+            # ra_find() handle the value.
+            if options['sizelimit'] > 0:
+                ra_options['sizelimit'] = options['sizelimit']
+        else:
+            ra_options['sizelimit'] = self.api.Backend.ldap2.size_limit
 
         result = collections.OrderedDict()
         complete = bool(ra_options)
@@ -1678,6 +1702,8 @@ class cert_find(Search, CertMethod):
                 ra_options['subject'] = hosts[0]
             elif len(users) == 1 and not services and not hosts:
                 ra_options['subject'] = users[0]
+        if 'status' in options:
+            ra_options['status'] = options.get('status')
 
         try:
             ca_enabled_check(self.api)
@@ -1724,6 +1750,11 @@ class cert_find(Search, CertMethod):
         return result, False, complete
 
     def _ldap_search(self, all, pkey_only, no_members, **options):
+        # defer import of the OpenSSL module to not affect the requests
+        # module which will use pyopenssl if this is available.
+        if sys.modules.get('OpenSSL.SSL', False) is None:
+            del sys.modules["OpenSSL.SSL"]
+        import OpenSSL.crypto
         ldap = self.api.Backend.ldap2
 
         filters = []
@@ -1782,18 +1813,21 @@ class cert_find(Search, CertMethod):
         ca_enabled = getattr(context, 'ca_enabled')
         for entry in entries:
             for attr in ('usercertificate', 'usercertificate;binary'):
-                for cert in entry.get(attr, []):
+                for der in entry.raw.get(attr, []):
+                    cert = OpenSSL.crypto.load_certificate(
+                        OpenSSL.crypto.FILETYPE_ASN1, der)
                     cert_key = self._get_cert_key(cert)
                     try:
                         obj = result[cert_key]
                     except KeyError:
-                        obj = {'serial_number': cert.serial_number}
+                        obj = {'serial_number': cert.get_serial_number()}
                         if not pkey_only and (all or not ca_enabled):
                             # Retrieving certificate details is now deferred
                             # until after all certificates are collected.
                             # For the case of CA-less we need to keep
                             # the certificate because getting it again later
                             # would require unnecessary LDAP searches.
+                            cert = cert.to_cryptography()
                             obj['certificate'] = (
                                 base64.b64encode(
                                     cert.public_bytes(x509.Encoding.DER))
@@ -1810,10 +1844,7 @@ class cert_find(Search, CertMethod):
 
     def execute(self, criteria=None, all=False, raw=False, pkey_only=False,
                 no_members=True, timelimit=None, sizelimit=None, **options):
-        # Store ca_enabled status in the context to save making the API
-        # call multiple times.
         ca_enabled = self.api.Command.ca_is_enabled()['result']
-        setattr(context, 'ca_enabled', ca_enabled)
 
         if 'cacn' in options:
             ca_obj = api.Command.ca_show(options['cacn'])['result']
@@ -1834,6 +1865,7 @@ class cert_find(Search, CertMethod):
             timelimit = self.api.Backend.ldap2.time_limit
         if sizelimit is None:
             sizelimit = self.api.Backend.ldap2.size_limit
+        options['sizelimit'] = sizelimit
 
         result = collections.OrderedDict()
         truncated = False
@@ -1888,7 +1920,9 @@ class cert_find(Search, CertMethod):
                         ca_obj = ca_objs[cacn] = (
                             self.api.Command.ca_show(cacn, all=True)['result'])
 
-                    obj.update(ra.get_certificate(str(serial_number)))
+                    obj.update(
+                        ra.get_certificate(serial_number)
+                    )
                     if not raw:
                         obj['certificate'] = (
                             obj['certificate'].replace('\r\n', ''))
@@ -1931,5 +1965,12 @@ class ca_is_enabled(Command):
     has_output = output.standard_value
 
     def execute(self, *args, **options):
-        result = is_service_enabled('CA', conn=self.api.Backend.ldap2)
+        # Store ca_enabled status in the context to save making the API
+        # call multiple times.
+        ca_enabled = getattr(context, 'ca_enabled', None)
+        if ca_enabled is not None and api.env.context in ('lite', 'server',):
+            result = ca_enabled
+        else:
+            result = is_service_enabled('CA', conn=self.api.Backend.ldap2)
+            setattr(context, 'ca_enabled', result)
         return dict(result=result, value=pkey_to_value(None, options))
