@@ -24,7 +24,7 @@ import logging
 
 from ipalib import api
 from ipalib import Int, Str, Flag
-from ipalib.constants import PATTERN_GROUPUSER_NAME
+from ipalib.constants import PATTERN_GROUPUSER_NAME, ERRMSG_GROUPUSER_NAME
 from ipalib.plugable import Registry
 from .baseldap import (
     add_external_post_callback,
@@ -40,7 +40,7 @@ from .baseldap import (
     LDAPRemoveMember,
     LDAPQuery,
 )
-from .idviews import remove_ipaobject_overrides
+from .idviews import remove_ipaobject_overrides, handle_idoverride_memberof
 from . import baseldap
 from ipalib import _, ngettext
 from ipalib import errors
@@ -52,7 +52,7 @@ if six.PY3:
 
 logger = logging.getLogger(__name__)
 
-if api.env.in_server and api.env.context in ['lite', 'server']:
+if api.env.in_server:
     try:
         import ipaserver.dcerpc
         _dcerpc_bindings_installed = True
@@ -70,6 +70,12 @@ converted to non-POSIX groups.
 
 Every group must have a description.
 
+The group name must follow these rules:
+- cannot contain only numbers
+- must start with a letter, a number, _ or .
+- may contain letters, numbers, _, ., or -
+- may end with a letter, a number, _, ., - or $
+
 POSIX groups must have a Group ID (GID) number. Changing a GID is
 supported but can have an impact on your file permissions. It is not necessary
 to supply a GID when creating a group. IPA will generate one automatically
@@ -157,6 +163,7 @@ Example:
 
 register = Registry()
 
+# also see "System: Remove Groups"
 PROTECTED_GROUPS = (u'admins', u'trust admins', u'default smb group')
 
 
@@ -196,6 +203,9 @@ class group(LDAPObject):
     object_class_config = 'ipagroupobjectclasses'
     possible_objectclasses = ['posixGroup', 'mepManagedEntry', 'ipaExternalGroup']
     permission_filter_objectclasses = ['posixgroup', 'ipausergroup']
+    permission_filter_objectclasses_string = (
+        '(|(objectclass=ipausergroup)(objectclass=posixgroup))'
+    )
     search_attributes_config = 'ipagroupsearchfields'
     default_attributes = [
         'cn', 'description', 'gidnumber', 'member', 'memberof',
@@ -204,10 +214,10 @@ class group(LDAPObject):
     ]
     uuid_attribute = 'ipauniqueid'
     attribute_members = {
-        'member': ['user', 'group', 'service'],
+        'member': ['user', 'group', 'service', 'idoverrideuser'],
         'membermanager': ['user', 'group'],
         'memberof': ['group', 'netgroup', 'role', 'hbacrule', 'sudorule'],
-        'memberindirect': ['user', 'group', 'service'],
+        'memberindirect': ['user', 'group', 'service', 'idoverrideuser'],
         'memberofindirect': ['group', 'netgroup', 'role', 'hbacrule',
         'sudorule'],
     }
@@ -249,7 +259,7 @@ class group(LDAPObject):
         'System: Modify Group Membership': {
             'ipapermright': {'write'},
             'ipapermtargetfilter': [
-                '(objectclass=ipausergroup)',
+                '(objectclass=ipausergroup)',  # only ipausergroups
                 '(!(cn=admins))',
             ],
             'ipapermdefaultattr': {'member'},
@@ -273,6 +283,10 @@ class group(LDAPObject):
         },
         'System: Modify Groups': {
             'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                permission_filter_objectclasses_string,
+                '(!(cn=admins))',
+            ],
             'ipapermdefaultattr': {
                 'cn', 'description', 'gidnumber', 'ipauniqueid',
                 'mepmanagedby', 'objectclass', 'membermanager',
@@ -284,6 +298,11 @@ class group(LDAPObject):
         },
         'System: Remove Groups': {
             'ipapermright': {'delete'},
+            'ipapermtargetfilter': [
+                permission_filter_objectclasses_string,
+                # prevent removal of PROTECTED_GROUPS
+                '(!(|(cn=admins)(cn=trust admins)(cn=default smb group)))',
+            ],
             'replaces': [
                 '(target = "ldap:///cn=*,cn=groups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Remove Groups";allow (delete) groupdn = "ldap:///cn=Remove Groups,cn=permissions,cn=pbac,$SUFFIX";)',
             ],
@@ -317,7 +336,7 @@ class group(LDAPObject):
     takes_params = (
         Str('cn',
             pattern=PATTERN_GROUPUSER_NAME,
-            pattern_errmsg='may only include letters, numbers, _, -, . and $',
+            pattern_errmsg=ERRMSG_GROUPUSER_NAME.format('group'),
             maxlength=255,
             cli_name='group_name',
             label=_('Group name'),
@@ -374,6 +393,15 @@ class group_add(LDAPCreate):
                 entry_attrs['gidnumber'] = baseldap.DNA_MAGIC
         return dn
 
+    def exc_callback(self, keys, options, exc, call_func,
+                     *call_args, **call_kwargs):
+        if isinstance(exc, errors.ObjectclassViolation):
+            if options['nonposix'] and 'gidnumber' in options:
+                raise errors.ObjectclassViolation(
+                    info=_('attribute "gidNumber" not allowed with --nonposix')
+                )
+        raise exc
+
 
 @register()
 class group_del(LDAPDelete):
@@ -477,6 +505,9 @@ class group_mod(LDAPUpdate):
             else:
                 old_entry_attrs['objectclass'].append('ipaexternalgroup')
                 entry_attrs['objectclass'] = old_entry_attrs['objectclass']
+            if 'gidnumber' in entry_attrs:
+                raise errors.MutuallyExclusiveError(reason=_(
+                    'An external group cannot be POSIX'))
 
         # Can't check for this in a validator because we lack context
         if 'gidnumber' in options and options['gidnumber'] is None:
@@ -523,6 +554,7 @@ class group_find(LDAPSearch):
         ),
     )
 
+    # pylint: disable-next=arguments-renamed
     def pre_callback(self, ldap, filter, attrs_list, base_dn, scope,
                      criteria=None, **options):
         assert isinstance(base_dn, DN)
@@ -593,6 +625,12 @@ class group_add_member(LDAPAddMember):
 
     takes_options = (ipaexternalmember_param,)
 
+    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
+        assert isinstance(dn, DN)
+        handle_idoverride_memberof(self, ldap, dn, found, not_found,
+                                   *keys, **options)
+        return dn
+
     def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
         assert isinstance(dn, DN)
         result = (completed, dn)
