@@ -94,6 +94,12 @@ usernames that start with a digit or usernames that exceed a certain length
 may cause problems for some UNIX systems.
 Use 'ipa config-mod' to change the username format allowed by IPA tools.
 
+The user name must follow these rules:
+- cannot contain only numbers
+- must start with a letter, a number, _ or .
+- may contain letters, numbers, _, ., or -
+- may end with a letter, a number, _, ., - or $
+
 
 EXAMPLES:
 
@@ -388,6 +394,19 @@ class stageuser_add(baseuser_add):
                 answer = self.api.Object['radiusproxy'].get_dn_if_exists(cl)
                 entry_attrs['ipatokenradiusconfiglink'] = answer
 
+        if 'ipaidpconfiglink' in entry_attrs:
+            cl = entry_attrs['ipaidpconfiglink']
+            if cl:
+                if 'objectclass' not in entry_attrs:
+                    _entry = ldap.get_entry(dn, ['objectclass'])
+                    entry_attrs['objectclass'] = _entry['objectclass']
+
+                if 'ipaidpuser' not in entry_attrs['objectclass']:
+                    entry_attrs['objectclass'].append('ipaidpuser')
+
+                answer = self.api.Object['idp'].get_dn_if_exists(cl)
+                entry_attrs['ipaidpconfiglink'] = answer
+
         self.pre_common_callback(ldap, dn, entry_attrs, attrs_list, *keys,
                                  **options)
 
@@ -560,9 +579,9 @@ class stageuser_activate(LDAPQuery):
 
         if self.obj.object_class_config:
             config = ldap.get_ipa_config()
-            entry_attrs['objectclass'] = config.get(
+            entry_attrs['objectclass'] = deepcopy(config.get(
                 self.obj.object_class_config, entry_attrs['objectclass']
-            )
+            ))
 
         return(entry_attrs)
 
@@ -575,7 +594,7 @@ class stageuser_activate(LDAPQuery):
         preserved (defined in preserved_DN_syntax_attrs)
         see http://www.freeipa.org/page/V3/User_Life-Cycle_Management#Adjustment_of_DN_syntax_attributes
         '''
-        if not attr in entry_to:
+        if attr not in entry_to:
             if isinstance(entry_from[attr], (list, tuple)):
                 # attr is multi value attribute
                 entry_to[attr] = []
@@ -720,7 +739,7 @@ class stageuser_activate(LDAPQuery):
         # Now delete the Staging entry
         try:
             self._exc_wrapper(args, options, ldap.delete_entry)(staging_dn)
-        except:
+        except BaseException:
             try:
                 logger.error("Fail to delete the Staging user after "
                              "activating it %s ", staging_dn)
