@@ -33,7 +33,7 @@ import copy
 import subprocess
 import tempfile
 import time
-from pipes import quote
+from shlex import quote
 import configparser
 from contextlib import contextmanager
 from pkg_resources import parse_version
@@ -66,6 +66,7 @@ from .env_config import env_to_script
 from .host import Host
 from .firewall import Firewall
 from .resolver import ResolvedResolver
+from .fips import is_fips_enabled, enable_crypto_subpolicy
 
 logger = logging.getLogger(__name__)
 
@@ -237,7 +238,7 @@ def fix_apache_semaphores(master):
                            raiseonerr=False)
 
     master.run_command(
-        'for line in `ipcs -s | grep apache ''| cut -d " " -f 2`; '
+        'for line in `ipcs -s | grep apache ''| cut -d " " -f 2`; ' +
         'do ipcrm -s $line; done', raiseonerr=False
     )
 
@@ -323,7 +324,8 @@ def set_default_ttl_for_ipa_dns_zone(host, raiseonerr=True):
 
 def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
                    extra_args=(), domain_level=None, unattended=True,
-                   external_ca=False, stdin_text=None, raiseonerr=True):
+                   external_ca=False, stdin_text=None, raiseonerr=True,
+                   random_serial=False):
     if domain_level is None:
         domain_level = host.config.domain_level
     check_domain_level(domain_level)
@@ -344,6 +346,10 @@ def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
         '-a', host.config.admin_password,
         "--domain-level=%i" % domain_level,
     ]
+
+    if random_serial:
+        args.append('--random-serial-numbers')
+
     if ipatests_dse:
         args.extend(["--dirsrv-config-file", ipatests_dse])
 
@@ -362,6 +368,8 @@ def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
     if setup_adtrust:
         args.append('--setup-adtrust')
         fw_services.append("freeipa-trust")
+        if is_fips_enabled(host):
+            enable_crypto_subpolicy(host, "AD-SUPPORT")
     if external_ca:
         args.append('--external-ca')
 
@@ -496,6 +504,8 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
     if setup_adtrust:
         args.append('--setup-adtrust')
         fw_services.append("freeipa-trust")
+        if is_fips_enabled(replica):
+            enable_crypto_subpolicy(replica, "AD-SUPPORT")
     if master_authoritative_for_client_domain(master, replica):
         args.extend(['--ip-address', replica.ip])
 
@@ -523,8 +533,8 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
 
 
 def install_client(master, client, extra_args=[], user=None,
-                   password=None, unattended=True, stdin_text=None,
-                   nameservers='master'):
+                   password=None, pkinit_identity=None, unattended=True,
+                   stdin_text=None, nameservers='master'):
     """
     :param nameservers: nameservers to write in resolver config. Possible
            values:
@@ -546,25 +556,30 @@ def install_client(master, client, extra_args=[], user=None,
         if nameservers == 'master':
             nameservers = master.ip
         client.resolver.setup_resolver(nameservers, master.domain.name)
-    if user is None:
-        user = client.config.admin_name
-    if password is None:
-        password = client.config.admin_password
 
     args = [
         'ipa-client-install',
         '--domain', client.domain.name,
         '--realm', client.domain.realm,
-        '-p', user,
-        '-w', password,
         '--server', master.hostname
     ]
 
+    if pkinit_identity:
+        args.extend(["--pkinit-identity", pkinit_identity])
+    else:
+        if user is None:
+            user = client.config.admin_name
+        if password is None:
+            password = client.config.admin_password
+        args.extend(['-p', user, '-w', password])
+
     if unattended:
         args.append('-U')
 
     args.extend(extra_args)
 
+    if is_fips_enabled(client) and getattr(master.config, 'ad_domains', False):
+        enable_crypto_subpolicy(client, "AD-SUPPORT")
     result = client.run_command(args, stdin_text=stdin_text)
 
     setup_sssd_conf(client)
@@ -579,12 +594,15 @@ def install_adtrust(host):
     Configures the compat tree for the legacy clients.
     """
     kinit_admin(host)
+    if is_fips_enabled(host):
+        enable_crypto_subpolicy(host, "AD-SUPPORT")
     host.run_command(['ipa-adtrust-install', '-U',
                       '--enable-compat',
                       '--netbios-name', host.netbios,
                       '-a', host.config.admin_password,
                       '--add-sids'])
 
+    host.run_command(['net', 'conf', 'setparm', 'global', 'log level', '10'])
     Firewall(host).enable_service("freeipa-trust")
 
     # Restart named because it lost connection to dirsrv
@@ -1332,7 +1350,9 @@ def two_connected_topo(master, replicas):
         yield v1, v3
         grow.append((v1, v3))
 
-        for (r, s) in grow:
+        i = 0
+        while i < len(grow):
+            r, s = grow[i]
             t = pool.pop(0)
 
             for (u, v) in [(r, t), (s, t)]:
@@ -1343,6 +1363,7 @@ def two_connected_topo(master, replicas):
                 yield v, x
                 yield w, x
                 grow.append((w, x))
+            i += 1
 
     except IndexError:
         return
@@ -1402,7 +1423,8 @@ def double_circle_topo(master, replicas, site_size=6):
 
 def install_topo(topo, master, replicas, clients, domain_level=None,
                  skip_master=False, setup_replica_cas=True,
-                 setup_replica_kras=False, clients_extra_args=()):
+                 setup_replica_kras=False, clients_extra_args=(),
+                 random_serial=False):
     """Install IPA servers and clients in the given topology"""
     if setup_replica_kras and not setup_replica_cas:
         raise ValueError("Option 'setup_replica_kras' requires "
@@ -1413,7 +1435,8 @@ def install_topo(topo, master, replicas, clients, domain_level=None,
         install_master(
             master,
             domain_level=domain_level,
-            setup_kra=setup_replica_kras
+            setup_kra=setup_replica_kras,
+            random_serial=random_serial,
         )
 
     add_a_records_for_hosts_in_master_domain(master)
@@ -1665,13 +1688,16 @@ def install_kra(host, domain_level=None,
 
 def install_ca(
         host, domain_level=None, first_instance=False, external_ca=False,
-        cert_files=None, raiseonerr=True, extra_args=()
+        cert_files=None, raiseonerr=True, extra_args=(),
+        random_serial=False,
 ):
     if domain_level is None:
         domain_level = domainlevel(host)
     check_domain_level(domain_level)
     command = ["ipa-ca-install", "-U", "-p", host.config.dirman_password,
                "-P", 'admin', "-w", host.config.admin_password]
+    if random_serial:
+        command.append('--random-serial-numbers')
     if not isinstance(extra_args, (tuple, list)):
         raise TypeError("extra_args must be tuple or list")
     command.extend(extra_args)
@@ -1901,7 +1927,7 @@ def ldappasswd_user_change(user, oldpw, newpw, master, use_dirman=False,
 
     if use_dirman:
         args = [paths.LDAPPASSWD, '-D',
-                str(master.config.dirman_dn),  # pylint: disable=no-member
+                str(master.config.dirman_dn),
                 '-w', master.config.dirman_password,
                 '-s', newpw, '-x', '-ZZ', '-H', master_ldap_uri, userdn]
     else:
@@ -1919,7 +1945,7 @@ def ldappasswd_sysaccount_change(user, oldpw, newpw, master, use_dirman=False):
 
     if use_dirman:
         args = [paths.LDAPPASSWD, '-D',
-                str(master.config.dirman_dn),  # pylint: disable=no-member
+                str(master.config.dirman_dn),
                 '-w', master.config.dirman_password,
                 '-a', oldpw,
                 '-s', newpw, '-x', '-ZZ', '-H', master_ldap_uri,
@@ -2055,6 +2081,25 @@ def group_add(host, groupname, extra_args=()):
     return host.run_command(cmd)
 
 
+def group_del(host, groupname):
+    cmd = [
+        "ipa", "group-del", groupname,
+    ]
+    return host.run_command(cmd)
+
+
+def group_add_member(host, groupname, users=None,
+                     raiseonerr=True, extra_args=()):
+    cmd = [
+        "ipa", "group-add-member", groupname
+    ]
+    if users:
+        cmd.append("--users")
+        cmd.append(users)
+    cmd.extend(extra_args)
+    return host.run_command(cmd, raiseonerr=raiseonerr)
+
+
 def ldapmodify_dm(host, ldif_text, **kwargs):
     """Run ldapmodify as Directory Manager
 
@@ -2086,8 +2131,7 @@ def ldapsearch_dm(host, base, ldap_args, scope='sub', **kwargs):
     args = [
         'ldapsearch',
         '-x', '-ZZ',
-        '-h', host.hostname,
-        '-p', '389',
+        '-H', "ldap://{}".format(host.hostname),
         '-D', str(host.config.dirman_dn),
         '-w', host.config.dirman_password,
         '-s', scope,
@@ -2146,6 +2190,17 @@ def create_active_user(host, login, password, first='test', last='user',
     kdestroy_all(host)
 
 
+def set_user_password(host, username, password):
+    temppass = "redhat\nredhat"
+    sendpass = f"redhat\n{password}\n{password}"
+    kdestroy_all(host)
+    kinit_admin(host)
+    host.run_command(["ipa", "passwd", username],stdin_text=temppass)
+    host.run_command(["kinit", username], stdin_text=sendpass)
+    kdestroy_all(host)
+    kinit_admin(host)
+
+
 def kdestroy_all(host):
     return host.run_command(['kdestroy', '-A'])
 
@@ -2259,7 +2314,8 @@ class KerberosKeyCopier:
        copier.copy_keys('/etc/krb5.keytab', tmpname, replacement=replacement)
     """
     host_princ_template = "host/{master}@{realm}"
-    valid_etypes = ['aes256-cts-hmac-sha1-96', 'aes128-cts-hmac-sha1-96']
+    valid_etypes = ['aes256-cts-hmac-sha384-192', 'aes128-cts-hmac-sha256-128',
+                    'aes256-cts-hmac-sha1-96', 'aes128-cts-hmac-sha1-96']
 
     def __init__(self, host):
         self.host = host
@@ -2442,6 +2498,18 @@ def get_platform(host):
     return result.stdout_text.strip()
 
 
+def get_platform_version(host):
+    result = host.run_command([
+        'python3', '-c',
+        'from ipaplatform.osinfo import OSInfo; print(OSInfo().version_number)'
+    ], raiseonerr=False)
+    assert result.returncode == 0
+    # stdout_text is a str in format "(X, Y)" and needs to be
+    # converted back to a functional tuple. This approach works with
+    # any number of version numbers filled, e.g. (34, ) or (8, 6) etc.
+    return tuple(map(int, re.findall(r'[0-9]+', result.stdout_text.strip())))
+
+
 def install_packages(host, pkgs):
     """Install packages on a remote host.
     :param host: the host where the installation takes place
@@ -2607,26 +2675,35 @@ def get_pki_version(host):
         raise ValueError("get_pki_version: pki is not installed")
 
 
-def get_healthcheck_version(host):
+def get_package_version(host, pkgname):
     """
-    Function to get healthcheck version on fedora and rhel
+    Get package version on remote host
     """
     platform = get_platform(host)
     if platform in ("rhel", "fedora"):
         cmd = host.run_command(
-            ["rpm", "-qa", "--qf", "%{VERSION}", "*ipa-healthcheck"]
+            ["rpm", "-qa", "--qf", "%{VERSION}", pkgname]
         )
-        healthcheck_version = cmd.stdout_text
-        if not healthcheck_version:
+        get_package_version = cmd.stdout_text
+        if not get_package_version:
             raise ValueError(
-                "get_healthcheck_version: "
-                "ipa-healthcheck package is not installed"
+                "get_package_version: "
+                "pkgname package is not installed"
             )
     else:
         raise ValueError(
-            "get_healthcheck_version: unknown platform %s" % platform
+            "get_package_version: unknown platform %s" % platform
         )
-    return healthcheck_version
+    return get_package_version
+
+
+def get_openldap_client_version(host):
+    """Get openldap-clients version on remote host"""
+    return get_package_version(host, 'openldap-clients')
+
+
+def get_healthcheck_version(host):
+    return get_package_version(host, '*ipa-healthcheck')
 
 
 def wait_for_ipa_to_start(host, timeout=60):
@@ -2649,6 +2726,24 @@ def wait_for_ipa_to_start(host, timeout=60):
             break
 
 
+def stop_ipa_server(host):
+    """Stop the entire IdM server using the ipactl utility.
+    """
+    host.run_command([paths.IPACTL, "stop"])
+
+
+def start_ipa_server(host):
+    """Start the entire IdM server using the ipactl utility
+    """
+    host.run_command([paths.IPACTL, "start"])
+
+
+def restart_ipa_server(host):
+    """Restart the entire IdM server using the ipactl utility
+    """
+    host.run_command([paths.IPACTL, "restart"])
+
+
 def dns_update_system_records(host):
     """Runs "ipa dns-update-system-records" on "host".
     """
