@@ -645,6 +645,9 @@ class TestIPACommand(IntegrationTest):
             (datetime.now() - timedelta(seconds=10)).timetuple()
         )
 
+        # Wait for sssd to be back online, hence test-user to become available
+        tasks.wait_for_sssd_domain_status_online(self.master)
+
         tasks.run_ssh_cmd(
             to_host=self.master.external_hostname, username=test_user,
             auth_method="key", private_key_path=first_priv_key_path
@@ -719,9 +722,13 @@ class TestIPACommand(IntegrationTest):
 
     def test_certificate_out_write_to_file(self):
         # commands to test; name of temporary file will be appended
+        result = self.master.run_command([
+            'openssl', 'x509', '-serial', '-noout', '-in', paths.IPA_CA_CRT
+        ])
+        serial = result.stdout_text.strip().split('=', maxsplit=1)[1]
         commands = [
-            ['ipa', 'cert-show', '1', '--certificate-out'],
-            ['ipa', 'cert-show', '1', '--chain', '--certificate-out'],
+            ['ipa', 'cert-show', serial, '--certificate-out'],
+            ['ipa', 'cert-show', serial, '--chain', '--certificate-out'],
             ['ipa', 'ca-show', 'ipa', '--certificate-out'],
             ['ipa', 'ca-show', 'ipa', '--chain', '--certificate-out'],
         ]
@@ -1469,6 +1476,29 @@ class TestIPACommand(IntegrationTest):
             assert 'This account is currently not available' in \
                 result.stdout_text
 
+    def test_ipa_cacert_manage_prune(self):
+        """Test for ipa-cacert-manage prune"""
+
+        certfile = os.path.join(self.master.config.test_dir, 'cert.pem')
+        self.master.put_file_contents(certfile, isrgrootx1)
+        result = self.master.run_command(
+            [paths.IPA_CACERT_MANAGE, 'install', certfile])
+
+        certs_before_prune = self.master.run_command(
+            [paths.IPA_CACERT_MANAGE, 'list'], raiseonerr=False
+        ).stdout_text
+
+        assert isrgrootx1_nick in certs_before_prune
+
+        # Jump in time to make sure the cert is expired
+        self.master.run_command(['date', '-s', '+15Years'])
+        result = self.master.run_command(
+            [paths.IPA_CACERT_MANAGE, 'prune'], raiseonerr=False
+        ).stdout_text
+        self.master.run_command(['date', '-s', '-15Years'])
+
+        assert isrgrootx1_nick in result
+
     def test_ipa_getkeytab_server(self):
         """
         Exercise the ipa-getkeytab server options
@@ -1619,6 +1649,80 @@ class TestIPACommandWithoutReplica(IntegrationTest):
             tasks.kinit_admin(self.master)
             self.master.run_command(['ipa', 'user-del', user])
 
+    @pytest.fixture
+    def cleanupgroups(self):
+        """Fixture to remove any groups added as part of the tests.
+
+           It isn't necessary to remove all groupss created.
+
+           Ignore all errors.
+        """
+        yield
+        for group in ["testgroup1", "testgroup2", "testgroup3"]:
+            try:
+                self.master.run_command(['ipa', 'group-del', group])
+            except Exception:
+                pass
+
+    def test_sequence_processing_ipaexternalgroup(self, cleanupgroups):
+        """Test for sequence processing failures
+
+        Issues have been found for group_add sequence processing with
+        server context. This test checks that groups have correct userclass
+        when external is set to true or false with group-add.
+
+        related: https://pagure.io/freeipa/issue/9349
+        """
+        user_code_script = textwrap.dedent("""
+            from ipalib import api, errors
+            api.bootstrap_with_global_options(context='server')
+            api.finalize()
+            api.Backend.ldap2.connect()
+            
+            api.Command["group_add"]("testgroup1", external=True)
+            api.Command["group_add"]("testgroup2", external=False)
+            result1 = api.Command["group_show"]("testgroup1", all=True)["result"] # noqa: E501
+            result2 = api.Command["group_show"]("testgroup2", all=True)["result"] # noqa: E501
+            print("'testgroup2' userclass: %s" % repr(result2["objectclass"]))
+        """)
+        self.master.put_file_contents("/tmp/reproducer1_code.py",
+                                      user_code_script)
+        result = self.master.run_command(['python3',
+                                          '/tmp/reproducer1_code.py'])
+        assert "ipaexternalgroup" not in result.stdout_text
+
+    def test_sequence_processing_nonposix_group(self, cleanupgroups):
+        """Test for sequence processing failures
+
+        Issues have been found for group_add sequence processing with
+        server context after creating a nonposix group. This test checks
+        that all following group_add calls to add posix groups calls are
+        not failing with missing attribute.
+
+        related: https://pagure.io/freeipa/issue/9349
+        """
+        user_code_script2 = textwrap.dedent("""
+            from ipalib import api, errors
+            api.bootstrap_with_global_options(context='server')
+            api.finalize()
+            api.Backend.ldap2.connect()
+
+            api.Command["group_add"]("testgroup1", nonposix=False)
+            try:
+                api.Command["group_add"]("testgroup2", nonposix=True)
+            except Exception as e:
+                print("testgroup2: %s" % e)
+            try:
+                api.Command["group_add"]("testgroup3", external=True)
+            except Exception as e:
+                print("testgroup3: %s" % e)
+        """)
+        self.master.put_file_contents("/tmp/reproducer2_code.py",
+                                      user_code_script2)
+        result = self.master.run_command(['python3',
+                                          '/tmp/reproducer2_code.py'])
+        assert "missing attribute" not in result.stdout_text
+
 
 class TestIPAautomount(IntegrationTest):
     @classmethod
