@@ -516,7 +516,8 @@ class TestInstallWithCA_KRA1(InstallTestBase1):
     @classmethod
     def install(cls, mh):
         tasks.install_master(cls.master, setup_dns=cls.master_with_dns,
-                             setup_kra=True)
+                             setup_kra=True,
+                             random_serial=cls.random_serial)
 
     def test_replica0_ipa_kra_install(self):
         tasks.install_kra(self.replicas[0], first_instance=False)
@@ -536,7 +537,8 @@ class TestInstallWithCA_DNS1(InstallTestBase1):
 
     @classmethod
     def install(cls, mh):
-        tasks.install_master(cls.master, setup_dns=cls.master_with_dns)
+        tasks.install_master(cls.master, setup_dns=cls.master_with_dns,
+                             random_serial=cls.random_serial)
 
     @pytest.mark.skipif(config.domain_level == DOMAIN_LEVEL_0,
                         reason='does not work on DOMAIN_LEVEL_0 by design')
@@ -584,9 +586,8 @@ class TestInstallWithCA_DNS3(CALessBase):
     """
 
     @pytest.mark.xfail(
-        osinfo.id == 'fedora' and osinfo.version_number >= (33,)
-        and osinfo.version_number < (35,),
-        reason='freeipa ticket 8700', strict=True)
+        osinfo.id == 'fedora' and osinfo.version_number >= (36,),
+        reason='freeipa ticket 9135', strict=True)
     @server_install_setup
     def test_number_of_zones(self):
         """There should be two zones: one forward, one reverse"""
