@@ -23,17 +23,21 @@
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
-#include <stdbool.h>
+#include "config.h"
 #include <stdio.h>
+#include <string.h>
 #include <time.h>
 #include <ctype.h>
 #include <fcntl.h>
+#include <syslog.h>
 #include <unistd.h>
-#include <nss.h>
-#include <nssb64.h>
-#include <hasht.h>
-#include <pk11pub.h>
 #include <errno.h>
+#if defined(USE_PWQUALITY)
+#include <pwquality.h>
+#endif
+#include <openssl/evp.h>
+#include <openssl/rand.h>
+#include <openssl/sha.h>
 #include "ipa_pwd.h"
 
 #define GENERALIZED_TIME_LENGTH 15
@@ -161,31 +165,23 @@ static int ipapwd_gentime_cmp(const void *p1, const void *p2)
 
 #define SHA_SALT_LENGTH 8
 
-/* SHA*_LENGTH leghts come from nss3/hasht.h */
-#define SHA_HASH_MAX_LENGTH SHA512_LENGTH
-
-static int ipapwd_hash_type_to_alg(char *hash_type,
-                                   SECOidTag *hash_alg,
-                                   unsigned int *hash_alg_len)
+static int ipapwd_hash_type_to_alg(const char *hash_type,
+                                   char **hash_alg_sn)
 {
     if (strncmp("{SSHA}", hash_type, 6) == 0) {
-        *hash_alg = SEC_OID_SHA1;
-        *hash_alg_len = SHA1_LENGTH;
+        *hash_alg_sn = SN_sha1;
         return 0;
     }
     if (strncmp("{SHA256}", hash_type, 8) == 0) {
-        *hash_alg = SEC_OID_SHA256;
-        *hash_alg_len = SHA256_LENGTH;
+        *hash_alg_sn = SN_sha256;
         return 0;
     }
     if (strncmp("{SHA384}", hash_type, 8) == 0) {
-        *hash_alg = SEC_OID_SHA384;
-        *hash_alg_len = SHA384_LENGTH;
+        *hash_alg_sn = SN_sha384;
         return 0;
     }
     if (strncmp("{SHA512}", hash_type, 8) == 0) {
-        *hash_alg = SEC_OID_SHA512;
-        *hash_alg_len = SHA512_LENGTH;
+        *hash_alg_sn = SN_sha512;
         return 0;
     }
 
@@ -202,8 +198,8 @@ static int ipapwd_hash_type_to_alg(char *hash_type,
 *
 * @return 0 on success, -1 on error.
 */
-static int ipapwd_hash_password(char *password,
-                                char *hash_type,
+int ipapwd_hash_password(char *password,
+                                const char *hash_type,
                                 unsigned char *salt,
                                 unsigned char **full_hash,
                                 unsigned int *full_hash_len)
@@ -213,47 +209,53 @@ static int ipapwd_hash_password(char *password,
     unsigned char *pwd = (unsigned char *)password;
     unsigned int pwdlen = strlen(password);
     unsigned char saltbuf[SHA_SALT_LENGTH];
-    unsigned char hash[SHA_HASH_MAX_LENGTH];
+    unsigned char hash[EVP_MAX_MD_SIZE] = { 0 };
     unsigned int hash_len;
-    SECOidTag hash_alg;
-    unsigned int hash_alg_len;
-    PK11Context *ctx = NULL;
+
     int ret;
 
-    NSS_NoDB_Init(".");
+    EVP_MD_CTX *ctx = NULL;
+    char *hash_alg_sn = NULL;
+    const EVP_MD* hash_alg = NULL;
 
     if (!salt) {
-        PK11_GenerateRandom(saltbuf, SHA_SALT_LENGTH);
+        RAND_bytes(saltbuf, SHA_SALT_LENGTH);
         salt = saltbuf;
     }
 
-    ret = ipapwd_hash_type_to_alg(hash_type, &hash_alg, &hash_alg_len);
+    ret = ipapwd_hash_type_to_alg(hash_type, &hash_alg_sn);
     if (ret) {
         return -1;
     }
 
-    ctx = PK11_CreateDigestContext(hash_alg);
+    hash_alg = EVP_get_digestbyname(hash_alg_sn);
+    if (!hash_alg) {
+        return -1;
+    }
+
+    ctx = EVP_MD_CTX_new();
     if (ctx == NULL) {
         return -1;
     }
 
-    memset(hash, 0, sizeof(hash));
-
-    ret = PK11_DigestBegin(ctx);
-    if (ret == SECSuccess) {
-        ret = PK11_DigestOp(ctx, pwd, pwdlen);
+    ret = EVP_DigestInit_ex(ctx, hash_alg, NULL);
+    if (ret) {
+        ret = EVP_DigestUpdate(ctx, pwd, pwdlen);
     }
-    if (ret == SECSuccess) {
-        ret = PK11_DigestOp(ctx, salt, SHA_SALT_LENGTH);
+    if (ret) {
+        ret = EVP_DigestUpdate(ctx, salt, SHA_SALT_LENGTH);
     }
-    if (ret == SECSuccess) {
-        ret = PK11_DigestFinal(ctx, hash, &hash_len, hash_alg_len);
+    if (ret) {
+        ret = EVP_DigestFinal_ex(ctx, hash, &hash_len);
     }
-    if (ret != SECSuccess) {
+    if (!ret) {
         ret = -1;
         goto done;
     }
-    if (hash_len != hash_alg_len) {
+    else
+        ret = 0;
+
+    if (hash_len != (unsigned int) EVP_MD_CTX_size(ctx)) {
         ret = -1;
         goto done;
     }
@@ -269,7 +271,7 @@ static int ipapwd_hash_password(char *password,
     memset(fh + fhl, '\0', 1);
 
 done:
-    PK11_DestroyContext(ctx, 1);
+    EVP_MD_CTX_free(ctx);
     *full_hash = fh;
     *full_hash_len = fhl;
     return ret;
@@ -291,14 +293,11 @@ static int ipapwd_cmp_password(char *password, char *historyString)
     char *hash_type;
     char *b64part;
     size_t b64_len;
-    SECItem *item;
     unsigned char *salt;
     unsigned char *hash = NULL;
     unsigned int hash_len;
     int ret;
 
-    NSS_NoDB_Init(".");
-
     hash_type = historyString;
     b64part = strchr(historyString, '}');
     if (!b64part) {
@@ -307,27 +306,29 @@ static int ipapwd_cmp_password(char *password, char *historyString)
     b64part++;
     b64_len = strlen(b64part);
 
-    item = NSSBase64_DecodeBuffer(NULL, NULL, b64part, b64_len);
-    if (!item) {
+    unsigned char *item_data = (unsigned char *) OPENSSL_malloc(sizeof(char) * b64_len);
+    if (!item_data) {
         return -1;
     }
-    if (item->len <= SHA_SALT_LENGTH) {
+
+    size_t item_len = EVP_DecodeBlock(item_data, (unsigned char *) b64part, b64_len);
+    if (item_len <= SHA_SALT_LENGTH) {
         ret = -1;
         goto done;
     }
 
-    salt = item->data + (item->len - SHA_SALT_LENGTH);
+    salt = item_data + (item_len - SHA_SALT_LENGTH);
     ret = ipapwd_hash_password(password, hash_type, salt, &hash, &hash_len);
     if (ret != 0) {
         goto done;
     }
 
-    if (hash_len != item->len) {
+    if (hash_len != item_len) {
         ret = 1;
         goto done;
     }
 
-    if (memcmp(item->data, hash, hash_len)) {
+    if (CRYPTO_memcmp(item_data, hash, hash_len)) {
         ret = 1;
         goto done;
     }
@@ -335,7 +336,9 @@ static int ipapwd_cmp_password(char *password, char *historyString)
     ret = 0;
 
 done:
-    SECITEM_FreeItem(item, 1);
+    if (item_data)
+        OPENSSL_clear_free(item_data, sizeof(char) * b64_len);
+
     free(hash);
     return ret;
 }
@@ -360,33 +363,33 @@ static char *ipapwd_hash_to_history(time_t hash_time,
 {
     struct tm utctime;
     char timestr[GENERALIZED_TIME_LENGTH+1];
-    SECItem item;
-    char *encoded;
-    char *history;
+    unsigned char *encoded = NULL;
+    char *history = NULL;
     int ret;
 
-    if (!gmtime_r(&hash_time, &utctime)) {
-        return NULL;
-    }
+    if (!gmtime_r(&hash_time, &utctime))
+        goto cleanup;
+
     strftime(timestr, GENERALIZED_TIME_LENGTH+1, "%Y%m%d%H%M%SZ", &utctime);
 
-    NSS_NoDB_Init(".");
+    encoded = (unsigned char *) malloc(4 * (hash_len * sizeof(*encoded) / 3) + 1);
+    if (!encoded)
+        goto cleanup;
 
-    item.type = siBuffer;
-    item.data = hash;
-    item.len = hash_len;
-
-    encoded = NSSBase64_EncodeItem(NULL, NULL, 0, &item);
-    if (!encoded) {
-        return NULL;
-    }
+    ret = EVP_EncodeBlock(encoded, hash, hash_len);
+    if (ret < 1)
+        goto cleanup;
 
     ret = asprintf(&history, "%s%s%s", timestr, hash_type, encoded);
     if (ret == -1) {
         history = NULL;
+        goto cleanup;
     }
 
-    free(encoded);
+cleanup:
+    if (encoded)
+        free(encoded);
+
     return history;
 }
 
@@ -405,6 +408,7 @@ static char *ipapwd_hash_to_history(time_t hash_time,
 */
 int ipapwd_check_policy(struct ipapwd_policy *policy,
                         char *password,
+                        char *user,
                         time_t cur_time,
                         time_t acct_expiration,
                         time_t pwd_expiration,
@@ -413,6 +417,13 @@ int ipapwd_check_policy(struct ipapwd_policy *policy,
 {
     int pwdlen, blen;
     int ret;
+#if defined(USE_PWQUALITY)
+    pwquality_settings_t *pwq;
+    int check_pwquality = 0;
+    int entropy = 0;
+    char buf[PWQ_MAX_ERROR_MESSAGE_LEN];
+    void *auxerror;
+#endif
 
     if (!policy || !password) {
         return IPAPWD_POLICY_ERROR;
@@ -461,7 +472,7 @@ int ipapwd_check_policy(struct ipapwd_policy *policy,
         char *p, *n;
         int size, len;
 
-        /* we want the actual lenght in bytes here */
+        /* we want the actual length in bytes here */
         len = blen;
 
         p = password;
@@ -525,6 +536,76 @@ int ipapwd_check_policy(struct ipapwd_policy *policy,
         }
     }
 
+#if defined(USE_PWQUALITY)
+    /* Only call into libpwquality if at least one setting is made
+     * because there are a number of checks that don't have knobs
+     * so preserve the previous behavior.
+     */
+    check_pwquality = policy->max_repeat + policy->max_sequence + policy->dictcheck + policy->usercheck;
+
+    if (check_pwquality > 0) {
+        /* Call libpwquality */
+        openlog(NULL, LOG_CONS | LOG_NDELAY, LOG_DAEMON);
+        pwq = pwquality_default_settings();
+        if (pwq == NULL) {
+            syslog(LOG_ERR, "Not able to set pwquality defaults\n");
+            return IPAPWD_POLICY_ERROR;
+        }
+        if (policy->min_pwd_length < 6)
+            syslog(LOG_WARNING, "password policy min length is < 6. Will be enforced as 6\n");
+        pwquality_set_int_value(pwq, PWQ_SETTING_MIN_LENGTH, policy->min_pwd_length);
+        pwquality_set_int_value(pwq, PWQ_SETTING_MAX_REPEAT, policy->max_repeat);
+        pwquality_set_int_value(pwq, PWQ_SETTING_MAX_SEQUENCE, policy->max_sequence);
+        pwquality_set_int_value(pwq, PWQ_SETTING_DICT_CHECK, policy->dictcheck);
+        pwquality_set_int_value(pwq, PWQ_SETTING_USER_CHECK, policy->usercheck);
+
+        entropy = pwquality_check(pwq, password, NULL, user, &auxerror);
+        pwquality_free_settings(pwq);
+
+#ifdef TEST
+        if (user != NULL) {
+            fprintf(stderr, "Checking password for %s\n", user);
+        } else {
+            fprintf(stderr, "No user provided\n");
+        }
+
+        fprintf(stderr, "min length %d\n", policy->min_pwd_length);
+        fprintf(stderr, "max repeat %d\n", policy->max_repeat);
+        fprintf(stderr, "max sequence %d\n", policy->max_sequence);
+        fprintf(stderr, "dict check %d\n", policy->dictcheck);
+        fprintf(stderr, "user check %d\n", policy->usercheck);
+#endif
+
+        if (entropy < 0) {
+#ifdef TEST
+            fprintf(stderr, "Bad password '%s': %s\n", password, pwquality_strerror(buf, sizeof(buf), entropy, auxerror));
+#endif
+            syslog(LOG_ERR, "Password is rejected with error %d: %s\n", entropy, pwquality_strerror(buf, sizeof(buf), entropy, auxerror));
+            switch (entropy) {
+            case PWQ_ERROR_MIN_LENGTH:
+                return IPAPWD_POLICY_PWD_TOO_SHORT;
+            case PWQ_ERROR_PALINDROME:
+                return IPAPWD_POLICY_PWD_PALINDROME;
+            case PWQ_ERROR_MAX_CONSECUTIVE:
+                return IPAPWD_POLICY_PWD_CONSECUTIVE;
+            case PWQ_ERROR_MAX_SEQUENCE:
+                return IPAPWD_POLICY_PWD_SEQUENCE;
+            case PWQ_ERROR_CRACKLIB_CHECK:
+                return IPAPWD_POLICY_PWD_DICT_WORD;
+            case PWQ_ERROR_USER_CHECK:
+                return IPAPWD_POLICY_PWD_USER;
+            default:
+                return IPAPWD_POLICY_PWD_COMPLEXITY;
+            }
+ 
+#ifdef TEST
+        } else {
+            fprintf(stderr, "Password '%s' is ok, entropy is %d\n", password, entropy);
+#endif
+        }
+    }
+#endif /* USE_PWQUALITY */
+
     if (pwd_history) {
         char *hash;
         int i;
@@ -548,13 +629,18 @@ char * IPAPWD_ERROR_STRINGS[] = {
     "Too soon to change password",
     "Password is too short",
     "Password reuse not permitted",
-    "Password is too simple"
+    "Password is too simple",
+    "Password has too many consecutive characters",
+    "Password contains a monotonic sequence",
+    "Password is based on a dictionary word",
+    "Password is a palindrone",
+    "Password contains username"
 };
 
 char * IPAPWD_ERROR_STRING_GENERAL = "Password does not meet the policy requirements";
 
 char * ipapwd_error2string(enum ipapwd_error err) {
-   if (err < 0 || err > IPAPWD_POLICY_PWD_COMPLEXITY) {
+   if (err < 0 || err > IPAPWD_POLICY_PWD_USER) {
        /* IPAPWD_POLICY_ERROR or out of boundary, return general error */
        return IPAPWD_ERROR_STRING_GENERAL;
    }
